@@ -1,6 +1,6 @@
 # ABSTRACT: create new Dancer2 application
 package Dancer2::CLI::Command::gen;
-$Dancer2::CLI::Command::gen::VERSION = '0.203001';
+$Dancer2::CLI::Command::gen::VERSION = '0.204000';
 use strict;
 use warnings;
 
@@ -216,8 +216,8 @@ sub _add_to_manifest_skip {
 sub _process_template {
     my ($template, $tokens) = @_;
     my $engine = Dancer2::Template::Simple->new;
-    $engine->{start_tag} = '[%';
-    $engine->{stop_tag} = '%]';
+    $engine->{start_tag} = '[d2%';
+    $engine->{stop_tag} = '%2d]';
     return $engine->render(\$template, $tokens);
 }
 
@@ -291,7 +291,7 @@ Dancer2::CLI::Command::gen - create new Dancer2 application
 
 =head1 VERSION
 
-version 0.203001
+version 0.204000
 
 =head1 AUTHOR
 
