@@ -1,6 +1,6 @@
 # ABSTRACT: encapsulation of Dancer2 packages
 package Dancer2::Core::App;
-$Dancer2::Core::App::VERSION = '0.203001';
+$Dancer2::Core::App::VERSION = '0.204000';
 use Moo;
 use Carp               qw<croak carp>;
 use Scalar::Util       'blessed';
@@ -11,9 +11,11 @@ use Sub::Quote;
 use File::Spec;
 use Module::Runtime    'use_module';
 
+use Plack::App::File;
 use Plack::Middleware::FixMissingBodyInRedirect;
 use Plack::Middleware::Head;
-use Plack::Middleware::Static;
+use Plack::Middleware::Conditional;
+use Plack::Middleware::ConditionalGET;
 
 use Dancer2::FileUtils 'path';
 use Dancer2::Core;
@@ -27,6 +29,12 @@ use Dancer2::Core::Factory;
 
 use Dancer2::Handler::File;
 
+our $EVAL_SHIM = sub {
+    my $code = shift;
+    $code->(@_);
+};
+
+
 # we have hooks here
 with qw<
     Dancer2::Core::Role::Hookable
@@ -424,9 +432,20 @@ sub _build_session {
 
         # if we have a session cookie, try to retrieve the session
         if ( defined $session_id ) {
-            eval  { $session = $engine->retrieve( id => $session_id ); 1; }
-            or do { $@ and $@ !~ /Unable to retrieve session/
-                        and croak "Fail to retrieve session: $@" };
+            eval  {
+                $EVAL_SHIM->(sub {
+                    $session = $engine->retrieve( id => $session_id );
+                });
+                1;
+            }
+            or do {
+                my $err = $@ || "Zombie Error";
+                if ( $err !~ /Unable to retrieve session/ ) {
+                    croak "Failed to retrieve session: $err"
+                } else {
+                    # XXX we throw away the error entirely? Why?
+                }
+            };
         }
     }
 
@@ -909,16 +928,23 @@ sub send_as {
             carp "Please use 'html' as the type for 'send_as', not $type";
         }
 
-        my $charset = $self->config->{charset} || 'UTF-8';
-        my $content = Encode::encode( $charset, $data );
-        $options->{content_type} ||= 'text/html; charset=' . $charset;
+        $options->{charset} = $self->config->{charset} || 'UTF-8';
+        my $content = Encode::encode( $options->{charset}, $data );
+        $options->{content_type} ||= 'text/html';
         $self->send_file( \$content, %$options );     # returns from sub
     }
 
     # Try and load the serializer class
     my $serializer_class = "Dancer2::Serializer::$type";
-    eval { require_module( $serializer_class ); 1; } or
-        croak "Unable to load serializer class for $type";
+    eval {
+        $EVAL_SHIM->(sub {
+            require_module( $serializer_class );
+        });
+        1;
+    } or do {
+        my $err = $@ || "Zombie Error";
+        croak "Unable to load serializer class for $type: $err";
+    };
 
     # load any serializer engine config
     my $engine_options =
@@ -953,7 +979,7 @@ sub send_file {
     my $thing   = shift;
     my %options = @_;
 
-    my ($content_type, $file_path);
+    my ($content_type, $charset, $file_path);
 
     # are we're given a filehandle? (based on what Plack::Middleware::Lint accepts)
     my $is_filehandle = Plack::Util::is_real_fh($thing)
@@ -1001,18 +1027,19 @@ sub send_file {
         # Read file content as bytes
         $fh = Dancer2::FileUtils::open_file( "<", $file_path );
         binmode $fh;
-
         $content_type = Dancer2::runner()->mime_type->for_file($file_path) || 'text/plain';
         if ( $content_type =~ m!^text/! ) {
-             $content_type .= "; charset=" . ( $self->config->{charset} || "utf-8" );
+            $charset = $self->config->{charset} || "utf-8";
         }
     }
 
     # Now we are sure we can render the file...
     $self->execute_hook( 'core.app.before_file_render', $file_path );
 
-    # response content type
+    # response content type and charset
     ( exists $options{'content_type'} ) and $content_type = $options{'content_type'};
+    ( exists $options{'charset'} ) and $charset = $options{'charset'};
+    $content_type .= "; charset=$charset" if $content_type and $charset;
     ( defined $content_type )
       and $self->response->header('Content-Type' => $content_type );
 
@@ -1116,11 +1143,12 @@ sub compile_hooks {
                 $Dancer2::Core::Route::RESPONSE->is_halted
                     and return;
 
-                eval  { $hook->(@_); 1; }
+                eval  { $EVAL_SHIM->($hook,@_); 1; }
                 or do {
+                    my $err = $@ || "Zombie Error";
                     $app->cleanup;
-                    $app->log('error', "Exception caught in '$position' filter: $@");
-                    croak "Exception caught in '$position' filter: $@";
+                    $app->log('error', "Exception caught in '$position' filter: $err");
+                    croak "Exception caught in '$position' filter: $err";
                 };
             };
 
@@ -1148,13 +1176,14 @@ sub lexical_prefix {
     # if the new prefix is empty, it's a meaningless prefix, just ignore it
     length $new_prefix and $self->prefix($new_prefix);
 
-    eval { $cb->() };
-    my $e = $@;
+    my $err;
+    my $ok= eval { $EVAL_SHIM->($cb); 1 }
+            or do { $err = $@ || "Zombie Error"; };
 
     # restore app prefix
     $self->prefix($app_prefix);
 
-    $e and croak "Unable to run the callback for prefix '$prefix': $e";
+    $ok or croak "Unable to run the callback for prefix '$prefix': $err";
 }
 
 sub add_route {
@@ -1261,46 +1290,18 @@ sub forward {
 
 # Create a new request which is a clone of the current one, apart
 # from the path location, which points instead to the new location
-# TODO this could be written in a more clean manner with a clone mechanism
 sub make_forward_to {
     my $self    = shift;
     my $url     = shift;
     my $params  = shift;
     my $options = shift;
 
-    my $request = $self->request;
-
-    # we clone the env to make sure we don't alter the existing one in $self
-    my $env = { %{ $request->env } };
-
-    $env->{PATH_INFO} = $url;
-
-    # request body fh has been read till end
-    # delete CONTENT_LENGTH in new request (no need to parse body again)
-    # and merge existing params
-    delete $env->{CONTENT_LENGTH};
-
-    my $new_request = Dancer2::Core::Request->new(
-        env         => $env,
-        body_params => {},
-    );
-
-    my $new_params = _merge_params( scalar( $request->params ), $params || {} );
-
+    my $overrides = { PATH_INFO => $url };
     exists $options->{method} and
-        $new_request->env->{'REQUEST_METHOD'} = $options->{method};
+        $overrides->{REQUEST_METHOD} = $options->{method};
 
-    # Copy params (these are already decoded)
-    $new_request->{_params}       = $new_params;
-    $new_request->{_body_params}  = $request->{_body_params};
-    $new_request->{_query_params} = $request->{_query_params};
-    $new_request->{_route_params} = $request->{_route_params};
-    $new_request->{body}          = $request->body;
-    $new_request->{headers}       = $request->headers;
-
-    # Copy remaining settings
-    $new_request->{is_behind_proxy} = $request->{is_behind_proxy};
-    $new_request->{vars}            = $request->{vars};
+    # "clone" the existing request
+    my $new_request = $self->request->_shallow_clone( $params, $overrides );
 
     # If a session object was created during processing of the original request
     # i.e. a session object exists but no cookie existed
@@ -1315,16 +1316,6 @@ sub make_forward_to {
     return $new_request;
 }
 
-sub _merge_params {
-    my $params = shift;
-    my $to_add = shift;
-
-    for my $key ( keys %$to_add ) {
-        $params->{$key} = $to_add->{$key};
-    }
-    return $params;
-}
-
 sub app { shift }
 
 # DISPATCHER
@@ -1361,13 +1352,14 @@ sub to_app {
 
         my $response;
         eval {
-            $response = $self->dispatch($env)->to_psgi;
+            $EVAL_SHIM->(sub{ $response = $self->dispatch($env)->to_psgi });
             1;
         } or do {
+            my $err = $@ || "Zombie Error";
             return [
                 500,
                 [ 'Content-Type' => 'text/plain' ],
-                [ "Internal Server Error\n\n$@"  ],
+                [ "Internal Server Error\n\n$err"  ],
             ];
         };
 
@@ -1378,14 +1370,19 @@ sub to_app {
     # FixMissingBodyInRedirect
     $psgi = Plack::Middleware::FixMissingBodyInRedirect->wrap( $psgi );
 
-    # Static content passes through to app on 404, conditionally applied.
-    # Construct the statis app to avoid a closure over $psgi
+    # Only add static content handler if requires
     if ( $self->config->{'static_handler'} ) {
-        $psgi = Plack::Middleware::Static->wrap(
-            $psgi,
-            path => sub { -f path( $self->config->{public_dir}, shift ) },
-            root => $self->config->{public_dir},
+        # Use App::File to "serve" the static content
+        my $static_app = Plack::App::File->new(
+            root         => $self->config->{public_dir},
             content_type => sub { $self->mime_type->for_name(shift) },
+        )->to_app;
+        # Conditionally use the static handler wrapped with ConditionalGET
+        # when the file exists. Otherwise the request passes into our app.
+        $psgi = Plack::Middleware::Conditional->wrap(
+            $psgi,
+            condition => sub { -f path( $self->config->{public_dir}, shift->{PATH_INFO} ) },
+            builder   => sub { Plack::Middleware::ConditionalGET->wrap( $static_app ) },
         );
     }
 
@@ -1544,17 +1541,28 @@ sub _dispatch_route {
     my ( $self, $route ) = @_;
 
     local $@;
-    eval { $self->execute_hook( 'core.app.before_request', $self ); 1; }
-        or return $self->response_internal_error($@);
+    eval {
+        $EVAL_SHIM->(sub {
+            $self->execute_hook( 'core.app.before_request', $self );
+        });
+        1;
+    } or do {
+        my $err = $@ || "Zombie Error";
+        return $self->response_internal_error($err);
+    };
     my $response = $self->response;
 
     if ( $response->is_halted ) {
         return $self->_prep_response( $response );
     }
 
-    $response = eval {
-        $route->execute($self)
-    } or return $self->response_internal_error($@);
+    eval {
+        $EVAL_SHIM->(sub{ $response = $route->execute($self) });
+        1;
+    } or do {
+        my $err = $@ || "Zombie Error";
+        return $self->response_internal_error($err);
+    };
 
     return $response;
 }
@@ -1625,7 +1633,7 @@ Dancer2::Core::App - encapsulation of Dancer2 packages
 
 =head1 VERSION
 
-version 0.203001
+version 0.204000
 
 =head1 DESCRIPTION
 
@@ -1796,6 +1804,33 @@ to make it work.
         my $app        = $WannaBeContext->app; # works
     };
 
+=head2 C< $SIG{__DIE__} > Compatibility via C< $Dancer2::Core::App::EVAL_SHIM >
+
+If an installation wishes to use C< $SIG{__DIE__} > hooks to enhance
+their error handling then it may be required to ensure that certain
+bookkeeping code is executed within every C<eval BLOCK> that Dancer2
+performs. This can be accomplished by overriding the global variable
+C<$Dancer2::Core::App::EVAL_SHIM> with a subroutine which does whatever
+logic is required.
+
+This routine must perform the equivalent of the following subroutine:
+
+    our $EVAL_SHIM = sub {
+        my $code = shift;
+        return $code->(@_);
+    };
+
+An example of overriding this sub might be as follows:
+
+    $Dancer2::Core::App::EVAL_SHIM = sub {
+        my $code = shift;
+        local $IGNORE_EVAL_COUNTER = $IGNORE_EVAL_COUNTER + 1;
+        return $code->(@_);
+    };
+
+B<Note:> that this is a GLOBAL setting, which must be set up before
+any form of dispatch or use of Dancer2.
+
 =head1 AUTHOR
 
 Dancer Core Developers
