@@ -1,6 +1,6 @@
 package Dancer2::Core::Request;
 # ABSTRACT: Interface for accessing incoming requests
-$Dancer2::Core::Request::VERSION = '0.203001';
+$Dancer2::Core::Request::VERSION = '0.204000';
 use strict;
 use warnings;
 use parent 'Plack::Request';
@@ -211,9 +211,8 @@ sub deserialize {
     # converts that to a PVIV. Some serializers are picky (JSON)..
     $self->{_body_params} = $data;
 
-    # Set body parameters (HMV)
-    # Not happy with fiddling with Plack::Request internals -- veryrusty Aug 2016.
-    $self->env->{'plack.request.body'} =
+    # Set body parameters (decoded HMV)
+    $self->{'body_parameters'} =
         Hash::MultiValue->from_mixed( ref $data eq 'HASH' ? %$data : () );
 
     return $data;
@@ -372,7 +371,7 @@ sub _set_route_parameters {
 sub body_parameters {
     my $self = shift;
     # defer to (the overridden) Plack::Request->body_parameters
-    $self->env->{'plack.request.body'} ||= _decode($self->SUPER::body_parameters());
+    $self->{'body_parameters'} ||= _decode($self->SUPER::body_parameters());
 }
 
 sub parameters {
@@ -384,7 +383,8 @@ sub parameters {
         return $self->$attr;
     }
 
-    $self->env->{'plack.request.merged'} ||= do {
+    # merge together the *decoded* parameters
+    $self->{'merged_parameters'} ||= do {
         my $query = $self->query_parameters;
         my $body  = $self->body_parameters;
         my $route = $self->route_parameters; # not in Plack::Request
@@ -617,6 +617,55 @@ sub _build_cookies {
     return $cookies;
 }
 
+# poor man's clone
+sub _shallow_clone {
+    my ($self, $params, $options) = @_;
+
+    # shallow clone $env; we don't want to alter the existing one
+    # in $self, then merge any overridden values
+    my $env = { %{ $self->env }, %{ $options || {} } };
+
+    # request body fh has been read till end
+    # delete CONTENT_LENGTH in new request (no need to parse body again)
+    # and merge existing params
+    delete $env->{CONTENT_LENGTH};
+
+    my $new_request = __PACKAGE__->new(
+        env         => $env,
+        body_params => {},
+    );
+
+    # Clone and merge query params
+    my $new_params = $self->params;
+    $new_request->{_query_params} = { %{ $self->{_query_params} || {} } };
+    $new_request->{query_parameters} = $self->query_parameters->clone;
+    for my $key ( keys %{ $params || {} } ) {
+        my $value = $params->{$key};
+        $new_params->{$key} = $value;
+        $new_request->{_query_params}->{$key} = $value;
+        $new_request->{query_parameters}->add( $key => $value );
+    }
+
+    # Copy params (these are already decoded)
+    $new_request->{_params}       = $new_params;
+    $new_request->{_body_params}  = $self->{_body_params};
+    $new_request->{_route_params} = $self->{_route_params};
+    $new_request->{body}          = $self->body;
+    $new_request->{headers}       = $self->headers;
+
+    # Copy remaining settings
+    $new_request->{is_behind_proxy} = $self->{is_behind_proxy};
+    $new_request->{vars}            = $self->{vars};
+
+    # Clone any existing decoded & cached body params. (GH#1116 GH#1269)
+    $new_request->{'body_parameters'} = $self->body_parameters->clone;
+
+    # Delete merged HMV parameters, allowing them to be reconstructed on first use.
+    delete $new_request->{'merged_parameters'};
+
+    return $new_request;
+}
+
 1;
 
 __END__
@@ -631,7 +680,7 @@ Dancer2::Core::Request - Interface for accessing incoming requests
 
 =head1 VERSION
 
-version 0.203001
+version 0.204000
 
 =head1 SYNOPSIS
 
