@@ -1,6 +1,6 @@
 package Dancer2::Plugin;
 # ABSTRACT: base class for Dancer2 plugins
-$Dancer2::Plugin::VERSION = '0.203001';
+$Dancer2::Plugin::VERSION = '0.204000';
 use strict;
 use warnings;
 
@@ -572,7 +572,7 @@ sub _exporter_expand_sub {
     $orig_cb and *{'Dancer2::Core::App::add_hook'} = sub {
         my ( $app, $hook ) = @_;
 
-        my $hook_code = $hook->code;
+        my $hook_code = Scalar::Util::blessed($hook) ? $hook->code : $hook->{code};
         my $plugin    = $CUR_PLUGIN;
 
         $hook->{'code'} = sub {
@@ -619,7 +619,7 @@ Dancer2::Plugin - base class for Dancer2 plugins
 
 =head1 VERSION
 
-version 0.203001
+version 0.204000
 
 =head1 SYNOPSIS
 
